/*-----------------------------------------------------------------------------
|   File:      CC1110_UART.c
+----------------------------------------------------------------------------*/

/*==== DECLARATION CONTROL ===================================================*/
/*==== INCLUDES ==============================================================*/

#include "hal_types.h"
#include "hal_board.h"
#include "hal_cc8051.h"
#include "CC1110_UART.h"

/*==== PUBLIC FUNCTIONS ======================================================*/

/*==== TYPES =================================================================*/
/*==== EXPORTS ===============================================================*/

/*==== MACROS=================================================================*/

//#define INIT_TP1()   do { TP1 = PIN_OFF; IO_DIR_PORT_PIN(1, 6, IO_OUT); P1SEL &= ~0x40;} while (0)

// U1CSR (0xF8) - USART 1 Control and Status - bit accessible SFR register
#define U1CSR_MODE                        0x80
#define U1CSR_RE                          0x40
#define U1CSR_SLAVE                       0x20
#define U1CSR_FE                          0x10
#define U1CSR_ERR                         0x08
#define U1CSR_RX_BYTE                     0x04
#define U1CSR_TX_BYTE                     0x02
#define U1CSR_ACTIVE                      0x01

// U1UCR (0xFB) - USART 1 UART Control
#define U1UCR_FLUSH                       0x80
#define U1UCR_FLOW                        0x40
#define U1UCR_D9                          0x20
#define U1UCR_BIT9                        0x10
#define U1UCR_PARITY                      0x08
#define U1UCR_SPB                         0x04
#define U1UCR_STOP                        0x02
#define U1UCR_START                       0x01

// U1GCR (0xFC) - USART 1 Generic Control
#define U1GCR_CPOL                        0x80
#define U1GCR_CPHA                        0x40
#define U1GCR_ORDER                       0x20
#define U1GCR_BAUD_E                      0x1F
#define U1GCR_BAUD_E0                     0x01
#define U1GCR_BAUD_E1                     0x02
#define U1GCR_BAUD_E2                     0x04
#define U1GCR_BAUD_E3                     0x08
#define U1GCR_BAUD_E4                     0x10

// PERCFG (0xF1) - Peripheral Control
#define PERCFG_T1CFG                      0x40
#define PERCFG_T3CFG                      0x20
#define PERCFG_T4CFG                      0x10
#define PERCFG_U1CFG                      0x02
#define PERCFG_U0CFG                      0x01

char g_string_hex[] = {"0123456789ABCDEF"};

// Baudrate = 115.2 kbps (U0BAUD.BAUD_M = 34, U0GCR.BAUD_E = 12)
#define UART_0_BAUD_M  47
#define UART_0_BAUD_E  12

// Baudrate = 9600 bps (U0BAUD.BAUD_M = 131, U0GCR.BAUD_E = 8)
#define UART_1_BAUD_M  148
#define UART_1_BAUD_E  8

/*==== PRIVATE FUNCTIONS =====================================================*/

static void uart_common_init(void)
{
    PERCFG = PERCFG_U0CFG | PERCFG_U1CFG;
}

/*==== PUBLIC FUNCTIONS ======================================================*/

void Uart1_Init(void)
{
    uart_common_init();

      // Configure relevant Port P0 pins for peripheral function:
      // P0SEL.SELP0_2/3/4/5 = 1 => RX = P0_2, TX = P0_3, CT = P0_4, RT = P0_5
    //  P0SEL |= BIT5 | BIT4 | BIT3 | BIT2;
      P1SEL |= BIT7 | BIT6 ;
    
      // TX = P1_6, RX = P1_7
    //  P0SEL |= BIT6 | BIT7;
    
    
      /***************************************************************************
       * Configure UART
       *
       * The system clock source used is the HS XOSC at 26 MHz speed.
       */
    
      // Set system clock source to 26 Mhz XSOSC to support maximum transfer speed,
      // ref. [clk]=>[clk_xosc.c]
    /*
      SLEEP &= ~SLEEP_OSC_PD;
      while( !(SLEEP & SLEEP_XOSC_S) );
      CLKCON = (CLKCON & ~(CLKCON_CLKSPD | CLKCON_OSC)) | CLKSPD_DIV_1;
      while (CLKCON & CLKCON_OSC);
      SLEEP |= SLEEP_OSC_PD;
    */
    
      // Initialise bitrate = 57.6 kbps (U0BAUD.BAUD_M = 34, U0GCR.BAUD_E = 11)
      U1BAUD = UART_1_BAUD_M;
      U1GCR = (U1GCR&~U1GCR_BAUD_E) | UART_1_BAUD_E;
   
      // Initialise UART protocol (start/stop bit, data bits, parity, etc.):
    
      // USART mode = UART (U0CSR.MODE = 1)
      U1CSR |= U1CSR_MODE;

    // Enable UART0 RX (U0CSR.RE = 1)
    U1CSR |= U1CSR_RE;

#if 0    
      // Start bit level = low => Idle level = high  (U0UCR.START = 0)
      U1UCR &= ~U1UCR_START;
    
      // Stop bit level = high (U0UCR.STOP = 1)
      U1UCR |= U1UCR_STOP;
    
      // Number of stop bits = 1 (U0UCR.SPB = 0)
      U1UCR &= ~U1UCR_SPB;
    
      // Parity = disabled (U0UCR.PARITY = 0)
      U1UCR &= ~U1UCR_PARITY;
    
      // 9-bit data enable = 8 bits transfer (U0UCR.BIT9 = 0)
//      U1UCR &= ~U1UCR_BIT9;
      U1UCR |= U1UCR_BIT9;
    
      // Level of bit 9 = 0 (U0UCR.D9 = 0), used when U0UCR.BIT9 = 1
      // Level of bit 9 = 1 (U0UCR.D9 = 1), used when U0UCR.BIT9 = 1
      // Parity = Even (U0UCR.D9 = 0), used when U0UCR.PARITY = 1
      // Parity = Odd (U0UCR.D9 = 1), used when U0UCR.PARITY = 1
      U1UCR &= ~U1UCR_D9;
    
      // Flow control = disabled (U0UCR.FLOW = 0)
      U1UCR &= ~U1UCR_FLOW;
#endif

      // Bit order = LSB first (U0GCR.ORDER = 0)
      U1GCR &= ~U1GCR_ORDER;
    //  U1GCR |= U1GCR_ORDER;
}

#if 0
void Uart1_Send_Char(char txChar)
{
    if( txChar == '\n') {
        Uart1_Send_Char('\r');
    }

    U1CSR &= ~U1CSR_TX_BYTE;
    U1DBUF = txChar;
    while(! (U1CSR&U1CSR_TX_BYTE) );
}

BOOL Uart1_Receive_Char(char * rxChar_p)
{
// Clear any pending RX interrupt request (set U0CSR.RX_BYTE = 0)
// U1CSR &= ~U1CSR_RX_BYTE;

    // Wait until data received (U0CSR.RX_BYTE = 1)
//    while( !(U1CSR&U1CSR_RX_BYTE) );
    if(!(U1CSR&U1CSR_RX_BYTE)) return FALSE;

    * rxChar_p = (char)U1DBUF;
    return TRUE;
}

void Uart1_Send_String(char * txChar_p)
{
    while((* txChar_p) != 0)
        Uart1_Send_Char((* txChar_p++));
}

void Uart1_Send_HexVal (unsigned int hexVal)
{
    char charString[11];
    unsigned int i = 0;

    charString[i] = '0';
    i ++;
    charString[i] = 'x';
    i ++;

    for(; i < 11; i ++)
    {
        charString[i] = g_string_hex[ (hexVal >> (28 - ((i-2)*4))) & 0xF ];
    }
    charString[10] = 0;

    Uart1_Send_String(charString);
}
#endif

void Uart0_Init(void)
{
    uart_common_init();

    // Configure relevant Port P0 pins for peripheral function:
    // P0SEL.SELP0_2/3/4/5 = 1 => RX = P0_2, TX = P0_3, CT = P0_4, RT = P0_5
    //  P0SEL |= BIT5 | BIT4 | BIT3 | BIT2;
    // TX = P1_6, RX = P1_7
    //  P0SEL |= BIT6 | BIT7;
//    P1SEL |= BIT2 | BIT3 | BIT4 | BIT5;
    P1SEL |= BIT4 | BIT5;
    
    // Initialise bitrate = 57.6 kbps (U0BAUD.BAUD_M = 34, U0GCR.BAUD_E = 11)
    U0BAUD = UART_0_BAUD_M;
    U0GCR = (U0GCR&~U1GCR_BAUD_E) | UART_0_BAUD_E;

    // Initialise UART protocol (start/stop bit, data bits, parity, etc.):

    // USART mode = UART (U0CSR.MODE = 1)
    U0CSR |= U1CSR_MODE;

    // Start bit level = low => Idle level = high  (U0UCR.START = 0)
    U0UCR &= ~U1UCR_START;

    // Stop bit level = high (U0UCR.STOP = 1)
    U0UCR |= U1UCR_STOP;

    // Number of stop bits = 1 (U0UCR.SPB = 0)
    U0UCR &= ~U1UCR_SPB;

    // Parity = disabled (U0UCR.PARITY = 0)
    U0UCR &= ~U1UCR_PARITY;

    // 9-bit data enable = 8 bits transfer (U0UCR.BIT9 = 0)
    U0UCR |= U1UCR_BIT9;

    // Level of bit 9 = 0 (U0UCR.D9 = 0), used when U0UCR.BIT9 = 1
    // Level of bit 9 = 1 (U0UCR.D9 = 1), used when U0UCR.BIT9 = 1
    // Parity = Even (U0UCR.D9 = 0), used when U0UCR.PARITY = 1
    // Parity = Odd (U0UCR.D9 = 1), used when U0UCR.PARITY = 1
    U0UCR &= ~U1UCR_D9;

    // Flow control = disabled (U0UCR.FLOW = 0)
    U0UCR &= ~U1UCR_FLOW;

    // Bit order = LSB first (U0GCR.ORDER = 0)
    U0GCR &= ~U1GCR_ORDER;
    // U0GCR |= U1GCR_ORDER;
}

void Uart0_Send_Char(char txChar)
{
    if( txChar == '\n') {
        Uart0_Send_Char('\r');
    }

    U0CSR &= ~U1CSR_TX_BYTE;
    U0DBUF = txChar;
    while(! (U0CSR&U1CSR_TX_BYTE) );
}

char Uart0_Receive_Char(void)
{
    char rxChar;

    // Enable UART0 RX (U0CSR.RE = 1)
    U0CSR |= U1CSR_RE;

    // Clear any pending RX interrupt request (set U0CSR.RX_BYTE = 0)
    U0CSR &= ~U1CSR_RX_BYTE;

    // Wait until data received (U0CSR.RX_BYTE = 1)
    while( !(U0CSR&U1CSR_RX_BYTE) );

    rxChar = (char)U0DBUF;
    Uart0_Send_Char(rxChar);

    // Read UART0 RX buffer
    return (rxChar);
}

void Uart0_Send_String(char * txChar_p)
{
    while((* txChar_p) != 0)
        Uart0_Send_Char((* txChar_p++));
}

void Uart0_Send_HexVal (unsigned int hexVal)
{
    char charString[11];
    unsigned int i = 0;

    charString[i] = '0';
    i ++;
    charString[i] = 'x';
    i ++;

    for(; i < 11; i ++)
    {
        charString[i] = g_string_hex[ (hexVal >> (28 - ((i-2)*4))) & 0xF ];
    }
    charString[10] = 0;

    Uart0_Send_String(charString);
}

void Uart0_Send_StringAndHexVal(char * txChar_p, unsigned int hexVal)
{
    Uart0_Send_String(txChar_p);
    Uart0_Send_HexVal(hexVal);
    Uart0_Send_Char('\n');
}

/*==== END OF FILE ==========================================================*/
